var dataArray = [];

var defaultZoomTime = 24*60*60*1000; // 1 day
var minZoom = -6; // 22 minutes 30 seconds
var maxZoom = 8; // ~ 8.4 months

var zoomLevel = 0;
var viewportEndTime = new Date();
var viewportStartTime = new Date();

var dateselectdiv = document.getElementById("dateselect");
var statusdiv = document.getElementById("status");

loadCSV(); // Download the CSV data, load Google Charts, parse the data, and draw the chart

function loadCSV()
{
    statusdiv.textContent = "Loading...";

    var xhr = new XMLHttpRequest();
    xhr.onreadystatechange = function() {
        if (this.readyState === 4 && this.status === 200) {
            dataArray = parseCSV(this.responseText);
            google.charts.load('current', { 'packages': ['line', 'corechart'] });
            google.charts.setOnLoadCallback(updateViewport);
        }
    };
    xhr.open("GET", "data.csv", true);
    xhr.send();
}

function parseCSV(string)
{
    var array = [];
    var lines = string.split("\n");
    for (var i = 0; i < lines.length; i++) {
        var data = lines[i].split(",", 2);
        data[0] = new Date(parseInt(data[0]) * 1000);
        data[1] = parseFloat(data[1]);
        array.push(data);
    }
    return array;
}

function drawChart()
{
    var data = new google.visualization.DataTable();

    data.addColumn('datetime', 'UNIX');
    data.addColumn('number', '°C');

    data.addRows(dataArray);

    console.log(document.body.clientHeight - dateselectdiv.clientHeight);

    var options = {
        curveType: 'function',
        height: 240,
        legend: { position: 'none' },
        hAxis: {
            viewWindow: {
                min: viewportStartTime,
                max: viewportEndTime
            },
            gridlines: {
                count: -1,
                units: {
                    days: { format: ['MMM dd'] },
                    hours: { format: ['HH:mm', 'ha'] },
                }
            },
            minorGridlines: {
                units: {
                    hours: { format: ['hh:mm:ss a', 'ha'] },
                    minutes: { format: ['HH:mm a Z', ':mm'] }
                }
            }
        },
        vAxis: {
            title: "Temperature °C"
        }
    };

    var chart = new google.visualization.LineChart(document.getElementById('chart_div'));
    chart.draw(data, options);
    
    dateselectdiv.removeAttribute("disabled");
}

// Display the start and end date on the page
function displayDate()
{
    var endDay = viewportEndTime.getDate();
    var endMonth = viewportEndTime.getMonth();
    var startDay = viewportStartTime.getDate();
    var startMonth = viewportStartTime.getMonth()
    
    if (endDay === startDay && endMonth === startMonth) {
        statusdiv.textContent = viewportEndTime.toDateString()
    } else {
        statusdiv.textContent = viewportStartTime.toDateString() + " - " + viewportEndTime.toDateString();
    }
}

document.getElementById("prev").onclick = function() {
    viewportEndTime = new Date(viewportEndTime.getTime() - getViewportWidthTime()/3); // move the viewport to the left for one third of its width (e.g. if the viewport width is 3 days, move one day back in time)
    updateViewport();
};
document.getElementById("next").onclick = function() {
    viewportEndTime = new Date(viewportEndTime.getTime() + getViewportWidthTime()/3); // move the viewport to the right for one third of its width (e.g. if the viewport width is 3 days, move one day into the future)
    updateViewport();
};

document.getElementById("zoomout").onclick = function() {
    zoomLevel += 1; // increment the zoom level (zoom out)
    if (zoomLevel > maxZoom) {
        zoomLevel = maxZoom;
    } else {
        updateViewport();
    }
};
document.getElementById("zoomin").onclick = function() {
    zoomLevel -= 1; // decrement the zoom level (zoom in)
    if (zoomLevel < minZoom) {
        zoomLevel = minZoom;
    } else {
        updateViewport();
    }
};

document.getElementById("reset").onclick = function() {
    viewportEndTime = new Date(); // the end time of the viewport is the current time
    zoomLevel = 0; // reset the zoom level to the default (one day)
    updateViewport();
};
document.getElementById("refresh").onclick = function() {
    viewportEndTime = new Date(); // the end time of the viewport is the current time
    loadCSV(); // download the latest data and re-draw the chart
};

document.body.onresize = drawChart;

function updateViewport() {
    viewportStartTime = new Date(viewportEndTime.getTime() - getViewportWidthTime());
    displayDate();
    drawChart();
}
function getViewportWidthTime() {
    return defaultZoomTime * (2 ** zoomLevel); // exponential relation between zoom level and zoom time span
                                               // every time you zoom, you double or halve the time scale
}